/*
 * MutableHttpResponse.java
 *
 * Created on January 2, 2007, 11:39 AM
 *
 * Copyright (C) 2006 Eric Sheridan
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.owasp.csrf.http;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.ByteArrayOutputStream;

import java.util.Collections;
import java.util.Enumeration;
import java.util.Map;
import java.util.HashMap;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletResponseWrapper;

/**
 *
 * @author esheridan
 */
public class MutableHttpResponse extends HttpServletResponseWrapper {
    
    private HttpServletResponse response = null;
    
    private ByteArrayOutputStream output = null;
    
    private FilterOutputStream stream = null;
    
    private PrintWriter writer = null;
    
    private Map<String, String> cookies = null;
    
    private int contentLength = -1;
    
    public MutableHttpResponse(HttpServletResponse response) {
        super(response);
        
        this.response = response;
        
        output = new ByteArrayOutputStream();
        stream = new FilterOutputStream(output);
        writer = new PrintWriter(stream);
        cookies = new HashMap<String, String>();
    }

    public void flushBuffer() throws IOException {
        output.flush();
        stream.flush();
        writer.flush();
    }
    
    public int getBufferSize() {
        return output.size();
    }
    
    public ServletOutputStream getOutputStream() {
        return stream;
    }
    
    public PrintWriter getWriter() throws IOException {
        return writer;
    }
    
    public boolean isCommitted() {
        return output.size() != 0;
    }
    
    public void reset() throws IllegalStateException {
        if(isCommitted()) {
            throw new IllegalStateException("the response has already been committed");
        }
        
        /** Clear any buffers **/
        output = new ByteArrayOutputStream();
        stream = new FilterOutputStream(output);
        writer = new PrintWriter(stream);
        
        /** Reset the status code and headers **/
        this.response.reset();
    }
    
    public void resetBuffer() throws IllegalStateException {
        if(isCommitted()) {
            throw new IllegalStateException("the response has already been committed");
        }
        
        /** Clear any buffers **/
        output = new ByteArrayOutputStream();
        stream = new FilterOutputStream(output);
        writer = new PrintWriter(stream);
    }
    
    public void setContentLength(int length) {
        this.contentLength = length;
        super.setContentLength(length);
    }

    public void addCookie(Cookie cookie) {
        cookies.put(cookie.getName(), cookie.getValue());
        super.addCookie(cookie);
    }

    public byte[] getContent() throws IOException {
        /**
         * FIXME: Why do we have to flush here? Why isnt flushBuffer() getting called ?
         */
        output.flush();
        stream.flush();
        writer.flush();
        return output.toByteArray();
    }
    
    public int getContentLength() {
        return contentLength;
    }
    
    public Enumeration getCookieNames() {
        return Collections.enumeration(cookies.keySet());
    }
    
    public String getCookieValue(String name) {
        return cookies.get(name);
    }
    
    public HttpServletResponse getResponse() {
        return response;
    }
    
    public FilterOutputStream getStream() {
        return stream;
    }
    
    public ByteArrayOutputStream getOutput() {
        return output;
    }
}
